﻿-- Author      : Nogarder
-- Create Date : 2012/3/16 18:32:11
-- get global functions for speed
local pairs = pairs
local table_getn=table.getn
local table_insert=table.insert
local type=type
local next=next
--
local NPA_EventHandler_FireEvent=NPA.EventHandler.FireEvent
--version control
local NPA_SettingManager_Version="A2"
---main
--module list
local NPA_SettingManager_DefaultFunc={}
--frame to key list
local NPA_SettingManager_ElementList={}
--framedata to profile list
local NPA_SettingManager_FrameDataToProfile={}
--resource type list
local ResourcesList={}
--
--function for registering profile
NPA.SettingManager.RegisterDefault=function(name,func)
	NPA_SettingManager_DefaultFunc[name]=func
end
--macro for get default setting
NPA.SettingManager.GetDefault=function(name)
	local entry=NPA_SettingManager_DefaultFunc[name]()
	--remove placeholder strings for non-compare function
	local checklist={
		[1]=entry
	}
	while(next(checklist))do
		for index,data in pairs(checklist)do
			if(type(data)=="table")then
				data.CheckWithSubString=nil
				data.DontCheck=nil
				for _,data2 in pairs(data)do
					if(type(data2)=="table")then
						table_insert(checklist,data2)
					end
				end
			end
			checklist[index]=nil
		end
	end
	--
	return entry
end
NPA.SettingManager.GetCompareDefault=function(name)
	return NPA_SettingManager_DefaultFunc[name]()
end
--macro for setting entry
local NPA_SettingManager_CompareSettingEntry=function(entry1,entry2)
	local checklist={
		[1]={
			EntryA=entry1,
			EntryB=entry2}
	}
	while(next(checklist))do
		for index,data in pairs(checklist)do
			local entryA=data.EntryA
			local entryB=data.EntryB
			--for everyone in entry2, make sure there's on in entry1
			for name,key in pairs(entryB) do
				--
				if(entryA[name]==nil or type(entryA[name])~=type(key))then
					if(type(key)=="table")then
						entryA[name]=CopyTable(key)
					else
						entryA[name]=key
					end
				elseif(type(key)=="table")then --table deep check
					if(key.CheckWithSubString)then --substring special checks
						--clear type entry
						entryA[name].CheckWithSubString=nil
						--check texturegroup with one default
						for _,dkey in pairs(entryA[name])do
							table_insert(checklist,{EntryA=dkey,EntryB=key[key.CheckWithSubString]})
						end
					elseif(key.DontCheck==true)then
						--ignore deep check
						entryA[name].DontCheck=nil
					else
						table_insert(checklist,{EntryA=entryA[name],EntryB=key})
					end
				end
			end
			--if there's not one in entry2, clear it from entry1
			for name,_ in pairs(entryA) do
				if(entryB[name]==nil)then
					entryA[name]=nil
				end
			end
			--remove it from list after check
			checklist[index]=nil
		end
	end
end
--macro for check setting parts
--core
local NPA_SettingManager_CheckCoreSetting=function()
	if(not NPA_Setting.Core)then
		NPA_Setting.Core={}
	end
	local defaultcoresetting=NPA.SettingManager.GetCompareDefault("Core")
	NPA_SettingManager_CompareSettingEntry(NPA_Setting.Core,defaultcoresetting)
	return
end
--nameplate
local NPA_SettingManager_CheckNamePlateSetting=function()
	--main entry
	if(not NPA_Setting.NamePlate)then
		NPA_Setting.NamePlate={
			["Default"]={
				TextSetting=NPA.SettingManager.GetDefault("NamePlateText"),
				IconSetting=NPA.SettingManager.GetDefault("NamePlateIcon"),
				IconGroupSetting=NPA.SettingManager.GetDefault("NamePlateIconGroup"),
				BarGroupSetting=NPA.SettingManager.GetDefault("NamePlateBarGroup"),
				TextureGroupSetting=NPA.SettingManager.GetDefault("NamePlateTextureGroup"),
				NumberSetting=NPA.SettingManager.GetDefault("NamePlateNumber"),
				DefaultSetting=NPA.SettingManager.GetDefault("NamePlateDefault"),
				AlphaSetting=NPA.SettingManager.GetDefault("NamePlateAlpha"),
			}
		}
		return
	end
	--
	local nameplatechecklist=	{
			TextSetting=NPA.SettingManager.GetDefault("NamePlateText"),
			IconSetting=NPA.SettingManager.GetDefault("NamePlateIcon"),
			IconGroupSetting=NPA.SettingManager.GetDefault("NamePlateIconGroup"),
			BarGroupSetting=NPA.SettingManager.GetDefault("NamePlateBarGroup"),
			TextureGroupSetting=NPA.SettingManager.GetDefault("NamePlateTextureGroup"),
			NumberSetting=NPA.SettingManager.GetDefault("NamePlateNumber"),
			AlphaSetting=NPA.SettingManager.GetDefault("NamePlateAlpha"),
	}
	--
	local checklistdefault={
		TextSetting=NPA.SettingManager.GetCompareDefault("NamePlateText")["NameText"],
		TextureGroupSetting=NPA.SettingManager.GetCompareDefault("NamePlateTextureGroup")["Background"],
		BarGroupSetting=NPA.SettingManager.GetCompareDefault("NamePlateBarGroup")["HealthBar"],
		IconSetting=NPA.SettingManager.GetCompareDefault("NamePlateIcon")["ClassIcon"],
		IconGroupSetting=NPA.SettingManager.GetCompareDefault("NamePlateIconGroup")["Debuff"],
		NumberSetting=NPA.SettingManager.GetCompareDefault("NamePlateNumber")["HealthNumber"],
		AlphaSetting=NPA.SettingManager.GetCompareDefault("NamePlateAlpha")[1],
	}
	--default is special
	local defaultdefaultsetting=NPA.SettingManager.GetCompareDefault("NamePlateDefault")
	--
	for _,profile in pairs(NPA_Setting.NamePlate)do
		for entryname,entry in pairs(nameplatechecklist)do
			--check by list
			if(not profile[entryname])then
				profile[entryname]=entry
			else
				for _,dentry in pairs(profile[entryname])do
					NPA_SettingManager_CompareSettingEntry(dentry,checklistdefault[entryname])
				end
			end
		end
		--check default alone
		local defaultsetting=profile.DefaultSetting
		if(not defaultsetting)then
			profile.DefaultSetting=NPA.SettingManager.GetDefault("NamePlateDefault")
		else
			NPA_SettingManager_CompareSettingEntry(defaultsetting,defaultdefaultsetting)
		end
	end
end
--target arrow
local NPA_SettingManager_CheckTargetArrowSetting=function()
	if(not NPA_Setting.TargetArrow)then
		NPA_Setting.TargetArrow={}
	end
	local defaulttargetarrowsetting=NPA.SettingManager.GetCompareDefault("TargetArrow")
	for _,setting in pairs(NPA_Setting.TargetArrow)do
		NPA_SettingManager_CompareSettingEntry(setting,defaulttargetarrowsetting)
	end
	return
end
--scrolltext
local NPA_SettingManager_CheckCombatTextSetting=function()
	if(not NPA_Setting.CombatText)then
		NPA_Setting.CombatText={}
	end
	--check condition
	if(not NPA_Setting.CombatText.Condition)then
		NPA_Setting.CombatText.Condition={}
	end
	local defaultcombattextconditionsetting=NPA.SettingManager.GetCompareDefault("CombatTextCondition")
	for _,setting in pairs(NPA_Setting.CombatText.Condition)do
		NPA_SettingManager_CompareSettingEntry(setting,defaultcombattextconditionsetting)
	end
	--check animation
	if(not NPA_Setting.CombatText.Animation)then
		NPA_Setting.CombatText.Animation={}
	end
	local defaultcombattextanimationsetting=NPA.SettingManager.GetCompareDefault("CombatTextAnimation")
	for _,setting in pairs(NPA_Setting.CombatText.Animation)do
		for index,data in pairs(setting)do
			NPA_SettingManager_CompareSettingEntry(data,defaultcombattextanimationsetting)
		end
	end
	--
	return
end
--fonts
local NPA_SettingManager_CheckFontSetting=function()
	if(not NPA_Setting.Font)then
		NPA_Setting.Font={}
	end
	local setting=NPA_Setting.Font
	local defaultfont=NPA.SettingManager.GetCompareDefault("Font")
	--check when exists
	for _,entry in pairs(setting)do
		NPA_SettingManager_CompareSettingEntry(entry,defaultfont)
	end
	--there must be a default
	if(not setting["Default"])then
		setting["Default"]=CopyTable(defaultfont)
	end
	ResourcesList["Font"]=NPA_Setting.Font
end
--textures
local NPA_SettingManager_CheckTextureSetting=function()
	if(not NPA_Setting.Texture)then
		NPA_Setting.Texture={}
	end
	local setting=NPA_Setting.Texture
	local default=NPA.SettingManager.GetCompareDefault("Texture")
	local defaulttexture=default["Default"]
	--check when exists
	for _,entry in pairs(setting)do
		NPA_SettingManager_CompareSettingEntry(entry,defaulttexture)
	end
	--there must be those default values
	for name,texturesetting in pairs(default)do
		if(not setting[name])then
			setting[name]=CopyTable(texturesetting)
		end
	end
	ResourcesList["Texture"]=NPA_Setting.Texture
end
--color
local NPA_SettingManager_CheckColorSetting=function()
	if(not NPA_Setting.Color)then
		NPA_Setting.Color={}
	end
	local defaultcolor=NPA.SettingManager.GetCompareDefault("Color")
	NPA_SettingManager_CompareSettingEntry(NPA_Setting.Color,defaultcolor)
end
--number
local NPA_SettingManager_CheckNumberSetting=function()
	if(not NPA_Setting.Number)then
		NPA_Setting.Number={}
	end
	local setting=NPA_Setting.Number
	local defaultnumber=NPA.SettingManager.GetCompareDefault("Number")
	for _,entry in pairs(setting)do
		NPA_SettingManager_CompareSettingEntry(entry,defaultnumber)
	end
	if(not setting["Default"])then
		setting["Default"]=CopyTable(defaultnumber)
	end
	ResourcesList["Number"]=NPA_Setting.Number
end
--bar group
local NPA_SettingManager_CheckBarGroupSetting=function()
	if(not NPA_Setting.BarGroup)then
		NPA_Setting.BarGroup={}
	end
	local defaultbargroup=NPA.SettingManager.GetCompareDefault("BarGroup")
	local defaultbar=defaultbargroup[1]
	local setting=NPA_Setting.BarGroup
	for _,group in pairs(setting)do
		for _,bar in pairs(group)do
			NPA_SettingManager_CompareSettingEntry(bar,defaultbar)
		end
	end
	--
	if(not setting["Default"])then
		setting["Default"]=CopyTable(defaultbargroup)
	end
	ResourcesList["BarGroup"]=NPA_Setting.BarGroup
end
--class group
local NPA_SettingManager_CheckClassSetting=function()
	if(not NPA_Setting.Class)then
		NPA_Setting.Class={}
	end
	local defaultclass=NPA.SettingManager.GetCompareDefault("Class")
	NPA_SettingManager_CompareSettingEntry(NPA_Setting.Class,defaultclass)
end
--spell lib
local NPA_SettingManager_CheckSpellLibSetting=function()
	if(not NPA_Setting.SpellLib)then
		NPA_Setting.SpellLib={}
	end
	--
	local defaultspell=NPA.SettingManager.GetCompareDefault("Spell")
	for _,spelldata in pairs(NPA_Setting.SpellLib)do
		NPA_SettingManager_CompareSettingEntry(spelldata,defaultspell)
	end
end
--raid target
local NPA_SettingManager_CheckRaidTargetSetting=function()
	if(not NPA_Setting.RaidTarget)then
		NPA_Setting.RaidTarget={}
	end
	--
	local defaultraidtarget=NPA.SettingManager.GetCompareDefault("RaidTarget")
	NPA_SettingManager_CompareSettingEntry(NPA_Setting.RaidTarget,defaultraidtarget)
end
--
local NPA_SettingManager_CheckNamePlateCondition=function()
	if(not NPA_Setting.NamePlateCondition)then
		NPA_Setting.NamePlateCondition=NPA.SettingManager.GetCompareDefault("NamePlateCondition")
	end
	--place holder
	local defaultcondition=NPA.SettingManager.GetCompareDefault("NamePlateCondition")[1]
	for _,conditiondata in pairs(NPA_Setting.NamePlateCondition)do
		NPA_SettingManager_CompareSettingEntry(conditiondata,defaultcondition)
	end
end
--profile
local NPA_SettingManager_CheckProfileSetting=function()
	if(not NPA_Profile)then
		NPA_Profile={}
	end
	return
end
--check existing profile with default
NPA.SettingManager.CheckSettingEntry=function()
	--set it to default if not exists (or old
	if((not NPA_Setting) or NPA_Setting.Version~=NPA_SettingManager_Version)then
		NPA_Setting={
			Version=NPA_SettingManager_Version,
			Locale="Default"
		}
	end
	--check settings parts with macro
	--core
	NPA_SettingManager_CheckCoreSetting()
	--resources
	NPA_SettingManager_CheckFontSetting()
	NPA_SettingManager_CheckTextureSetting()
	NPA_SettingManager_CheckColorSetting()
	NPA_SettingManager_CheckNumberSetting()
	NPA_SettingManager_CheckBarGroupSetting()
	NPA_SettingManager_CheckClassSetting()
	NPA_SettingManager_CheckSpellLibSetting()
	NPA_SettingManager_CheckRaidTargetSetting()
	--NamePlate apperance
	NPA_SettingManager_CheckNamePlateSetting()
	--nameplate condition
	NPA_SettingManager_CheckNamePlateCondition()
	--target arrow
	NPA_SettingManager_CheckTargetArrowSetting()
	--combat text
	NPA_SettingManager_CheckCombatTextSetting()
	--profile
	NPA_SettingManager_CheckProfileSetting()
end
local OnProfileLoaded=function()
		--check setting
	NPA_Setting=NPA_Setting or {}
	--check setting entry format ,for compatibility 
	NPA.SettingManager.CheckSettingEntry()
	--fire on load event
	NPA.EventHandler.FireEvent(6,"SettingManager_SettingLoaded")
	--also fire a condition update event
	NPA.EventHandler.FireEvent(6,"NamePlate_ConditionUpdated")
	--also fire a targetarrow update event
	NPA.EventHandler.FireEvent(6,"TargetArrow_SettingUpdated")
	--and combattext update event
	NPA.EventHandler.FireEvent(6,"CombatText_SettingUpdated")
end
--profile on change event
NPA.EventHandler.RegisterEvent(6,"ProfileManager_ProfileLoaded","NPA_SettingManager",OnProfileLoaded)
--register variables onload event
NPA.EventHandler.RegisterEvent(0,"VARIABLES_LOADED","NPA_SettingManager",function()
	--unregister onload event
	NPA.EventHandler.RegisterEvent(0,"VARIABLES_LOADED","NPA_SettingManager",nil)
	--
	OnProfileLoaded()
end)
--macro for get resources data
NPA.SettingManager.GetResources=function(type,name)
	local entry=ResourcesList[type][name or "Default"]
	return entry or ResourcesList[type]["Default"]
end
--function for return profile
NPA.SettingManager.GetNamePlateProfileByName=function(name)
	return NPA_Setting.NamePlate[name or "Default"] or NPA_Setting.NamePlate["Default"]
end
--function for generating framegroup
NPA.SettingManager.GenerateFrameGroup=function(profilename)
	local profile=NPA.SettingManager.GetNamePlateProfileByName(profilename)
	local framegroup={}
	--text
	local textsetting=profile.TextSetting
	for name,_ in pairs(textsetting)do
		framegroup[name]=true
	end
	--texturegroup
	local texturegroupsetting=profile.TextureGroupSetting
	for name,entry in pairs(texturegroupsetting)do
		framegroup[name]=true
	end
	--icongroup
	--icon
	local iconsetting=profile.IconSetting
	for name,entry in pairs(iconsetting)do
		framegroup[name]=true
	end
	--bargroup
	local bargroupsetting=profile.BarGroupSetting
	for name,entry in pairs(bargroupsetting)do
		framegroup[name]=true
	end
	--number
	local numbersetting=profile.NumberSetting
	for name,entry in pairs(numbersetting)do
		framegroup[name]=true
	end
	return framegroup
end